package ch.uzh.csg.filesharing;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileReader;
import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.URISyntaxException;
import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

import ch.uzh.csg.filesharing.fastss.FileSharingFastSS;
import ch.uzh.csg.filesharing.fastss.PreviewURN;
import ch.uzh.csg.tomp2p.Replication;
import ch.uzh.csg.tomp2p.Utils;
import ch.uzh.csg.tomp2p.connection.BindInformation;
import ch.uzh.csg.tomp2p.connection.BindInformation.Protocol;

public class AFastSSDummy1
{
	private static final int MAX_CHARS = 200;

	public static void main(String[] args) throws IOException, URISyntaxException
	{
		File path = new File("/home/draft/workspace/AFastSS/Articles/laptop2");
		FileSharingFastSS fss[] = new FileSharingFastSS[1];
		for (int i = 0; i < fss.length; i++)
		{
			BindInformation bind = new BindInformation();
			bind.addListenProtocolHint(Protocol.IPv4);
			bind.addListenInterfaceHint("wlan0");
			fss[i] = new FileSharingFastSS(Utils.makeSHAHash("" + (i + 140)), 4000, 4000, bind,
					new Replication(), new File("/home/draft/.afastss"));
			fss[i].bootstrap(new InetSocketAddress(InetAddress.getByName("192.168.2.50"), 4000));
		}
		File[] flist = path.listFiles(new FileFilter()
		{
			@Override
			public boolean accept(File pathname)
			{
				return pathname.isFile();
			}
		});
		boolean first = false;
		while (true)
		{
			for (File f : flist)
			{
				publish(fss[0], f, first);
				if (!first)
					Utils.sleep(5000);
			}
			first = false;
		}
	}

	private static void publish(FileSharingFastSS fss, File f, boolean fast) throws IOException,
			URISyntaxException
	{
		String fileContent = readFileAsString(f);
		String title, preview;
		List<String> keywords;
		if (f.getName().endsWith(".xml"))
		{
			title = extractTitle(fileContent);
			String content = extractContent(fileContent);
			preview = content.substring(0, 100).replace("\n", " ");
			keywords = getKeywords(title + " " + content);
		}
		else
		{
			title = f.getName();
			preview = "Size: " + f.length() + " Date: " + new Date(f.lastModified());
			keywords = getKeywords(f.getName());
		}
		//
		PreviewURN purn = new PreviewURN(f, title, preview);
		fss.addFile(f, purn);
		fss.publish(purn, keywords, fast, new AtomicBoolean(true));
	}

	public static List<String> getKeywords(String content)
	{
		int index = content.indexOf("==");
		if (index >= 0)
			return AFastSSDummy.getKeywords1(content.substring(0, Math.min(index, MAX_CHARS)));
		else
			return AFastSSDummy.getKeywords1(content.substring(0, Math.min(content.length(),
					MAX_CHARS)));
	}

	private static String extractText(String fileContent)
	{
		String beginStr1 = "<text"; // Beginning tag of text
		String beginStr2 = ">";
		String endStr = "</text>"; // Endtag of text
		// String endStr = "Anker";
		int t = fileContent.indexOf(beginStr1);
		int s = fileContent.indexOf(beginStr2, t) + 1;
		int e = fileContent.indexOf(endStr);
		if ((s != -1) || (e != -1))
			return fileContent.substring(s, e).trim();
		else
			return "Error while parsing: String indexes out of bounds";
	}

	private static String extractContent(String fileContent)
	{
		String result = extractText(fileContent);
		// WikiModel wikiModel = new WikiModel("", "");
		// String htmlStr = wikiModel.render(result);
		String htmlStr = cleanHTMLTags(result);
		return htmlStr;
	}

	private static String extractTitle(String fileContent)
	{
		// Log.i(TAG, "-------------Starting extracting title--------------");
		String beginStr = "<title>";
		String endStr = "</title>";
		int s = fileContent.indexOf(beginStr) + beginStr.length();
		int e = fileContent.indexOf(endStr);
		// Log.i(TAG, "-------------Finished extracting title--------------");
		if ((s != -1) && (e != -1))
			return fileContent.substring(s, e).trim();
		else
			return "String indexes out of bounds";
	}

	private static String cleanHTMLTags(String htmlString)
	{
		return htmlString.replaceAll("\\<.*?\\>", "");
	}

	private static String readFileAsString(File file) throws java.io.IOException
	{
		StringBuffer fileData = new StringBuffer(1000);
		BufferedReader reader = new BufferedReader(new FileReader(file));
		char[] buf = new char[1024];
		int numRead = 0;
		while ((numRead = reader.read(buf)) != -1)
		{
			String readData = String.valueOf(buf, 0, numRead);
			fileData.append(readData);
			buf = new char[1024];
		}
		reader.close();
		return fileData.toString();
	}
}
