package ch.uzh.csg.filesharing;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

import org.apache.mina.core.service.IoAcceptor;
import org.apache.mina.core.service.IoHandlerAdapter;
import org.apache.mina.core.session.IdleStatus;
import org.apache.mina.core.session.IoSession;
import org.apache.mina.filter.codec.ProtocolCodecFilter;
import org.apache.mina.filter.codec.textline.TextLineCodecFactory;
import org.apache.mina.transport.socket.nio.NioSocketAcceptor;

import ch.uzh.csg.tomp2p.Utils;

public class Coordinator extends IoHandlerAdapter
{
	private final Map<SocketAddress, IoSession> remote = new ConcurrentHashMap<SocketAddress, IoSession>();
	private final Map<String, AtomicInteger> results = new ConcurrentHashMap<String, AtomicInteger>();
	private static IoAcceptor acceptor;
	private static boolean quit = false;
	private static String[] schemes = { "PSH2", "PSHR", "TFT" };
	private final static Random rnd = new Random();

	public static void main(String[] args) throws IOException
	{
		NODES = Integer.parseInt(args[1]);
		acceptor = new NioSocketAcceptor();
		acceptor.getFilterChain().addLast("codec",
				new ProtocolCodecFilter(new TextLineCodecFactory()));
		acceptor.setHandler(new Coordinator());
		acceptor.bind(new InetSocketAddress(Integer.parseInt(args[0])));
	}

	@Override
	protected void finalize() throws Throwable
	{
		System.err.println("exxx");
		acceptor.unbind();
		System.err.println("exxx 1");
		acceptor.dispose();
		System.err.println("exxx 2");
		super.finalize();
	}

	public Coordinator()
	{
		new Thread(new Runnable()
		{
			@Override
			public void run()
			{
				init();
			}
		}).start();
	}
	private static int NODES = 12;
	private static int MUL = 3;
	private static int DWN = 3;

	private void testrun()
	{
		// testruns
		for (int i = 420 / NODES; i <= 660 / NODES; i += 240 / NODES)
		{
			for (String scheme : schemes)
			{
				sendAll("-scheme " + scheme
						+ " -freeriders 0.0 -limit 8000 -credit 2000 -filemul 3 -nodes " + i
						+ " -machines " + NODES + " -goodwill 0.1 -downloads 3 -ttlPSH 10", true);
				waitFor("done boot", NODES);
				sendAll("sim", false);
				waitFor("done sim", NODES);
			}
		}
		for (int i = 420 / NODES; i <= 660 / NODES; i += 240 / NODES)
		{
			for (String scheme : schemes)
			{
				sendAll("-scheme " + scheme
						+ " -freeriders 0.0 -limit 6000 -credit 2000 -filemul 3 -nodes " + i
						+ " -machines " + NODES + " -goodwill 0.1 -downloads 3 -ttlPSH 10", true);
				waitFor("done boot", NODES);
				sendAll("sim", false);
				waitFor("done sim", NODES);
			}
		}
		for (int i = 420 / NODES; i <= 660 / NODES; i += 240 / NODES)
		{
			for (String scheme : schemes)
			{
				sendAll("-scheme " + scheme
						+ " -freeriders 0.0 -limit 6000 -credit 2000 -filemul 10 -nodes " + i
						+ " -machines " + NODES + " -goodwill 0.1 -downloads 3 -ttlPSH 10", true);
				waitFor("done boot", NODES);
				sendAll("sim", false);
				waitFor("done sim", NODES);
			}
		}
	}

	private void init()
	{
		System.err.println("wait for " + NODES + " nodes to connect");
		waitFor("init", NODES);
		// test run
		// if (true)
		// {
		// testrun();
		// return;
		// }
		// node change
		for (int i = 420 / NODES; i <= 1020 / NODES; i += 60 / NODES)
		{
			for (String scheme : schemes)
			{
				sendAll("-scheme " + scheme + " -freeriders 0.0 -limit 8000 -credit 2000 -filemul "
						+ MUL + " -nodes " + i + " -machines " + NODES
						+ " -goodwill 0.1 -downloads " + DWN + " -ttlPSH 10", true);
				waitFor("done boot", NODES);
				sendAll("sim", false);
				waitFor("done sim", NODES);
			}
		}
		// freeriders change
		for (int i = 0; i < 10; i++)
		{
			for (String scheme : schemes)
			{
				sendAll("-scheme " + scheme + " -freeriders 0." + i
						+ " -limit 8000 -credit 2000 -filemul " + MUL + " -nodes " + (510 / NODES)
						+ " -machines " + NODES + " -goodwill 0.1 -downloads " + DWN
						+ " -ttlPSH 10", true);
				waitFor("done boot", NODES);
				sendAll("sim", false);
				waitFor("done sim", NODES);
			}
		}
		// credit limit change
		for (int i = 2000; i <= 12000; i += 2000)
		{
			for (String scheme : schemes)
			{
				sendAll("-scheme " + scheme + " -freeriders 0.0 -limit " + i
						+ " -credit 2000 -filemul " + MUL + " -nodes " + (510 / NODES)
						+ " -machines " + NODES + " -goodwill 0.1 -downloads " + DWN
						+ " -ttlPSH 10", true);
				waitFor("done boot", NODES);
				sendAll("sim", false);
				waitFor("done sim", NODES);
			}
		}
		// split size change
		for (int i = 2000; i <= 12000; i += 2000)
		{
			for (String scheme : schemes)
			{
				sendAll("-scheme " + scheme + " -freeriders 0.0 -limit " + (i * 4) + " -credit "
						+ i + " -filemul " + MUL + " -nodes " + (510 / NODES) + " -machines "
						+ NODES + " -goodwill 0.1 -downloads " + DWN + " -ttlPSH 10", true);
				waitFor("done boot", NODES);
				sendAll("sim", false);
				waitFor("done sim", NODES);
			}
		}
		// mul
		for (int i = 1; i < 10; i++)
		{
			for (String scheme : schemes)
			{
				sendAll("-scheme " + scheme + " -freeriders 0.0 -limit 8000 -credit 2000 -filemul "
						+ i + " -nodes " + (510 / NODES) + " -machines " + NODES
						+ " -goodwill 0.1 -downloads " + DWN + " -ttlPSH 10", true);
				waitFor("done boot", NODES);
				sendAll("sim", false);
				waitFor("done sim", NODES);
			}
		}
		// dwn
		for (int i = 1; i < 10; i++)
		{
			for (String scheme : schemes)
			{
				sendAll("-scheme " + scheme + " -freeriders 0.0 -limit 8000 -credit 2000 "
						+ "-filemul 3 -nodes " + (510 / NODES) + " -machines " + NODES
						+ " -goodwill 0.1 -downloads " + i + " -ttlPSH 10", true);
				waitFor("done boot", NODES);
				sendAll("sim", false);
				waitFor("done sim", NODES);
			}
		}
		// goodwill
		for (double i = 0; i < 0.55; i += 0.05)
		{
			for (String scheme : schemes)
			{
				sendAll("-scheme " + scheme + " -freeriders 0.0 -limit 8000 -credit 2000 -filemul "
						+ MUL + " -nodes " + (510 / NODES) + " -machines " + NODES + " -goodwill "
						+ i + " -downloads " + DWN + " -ttlPSH 10", true);
				waitFor("done boot", NODES);
				sendAll("sim", false);
				waitFor("done sim", NODES);
			}
		}
		sendAll("exit", false);
	}

	private void sendAll(String string, boolean append)
	{
		int i = 0;
		// List<IoSession> list = new ArrayList<IoSession>(remote.values());
		// while (list.size() > 0)
		for (IoSession session : remote.values())
		{
			// IoSession session = list.get(rnd.nextInt(list.size()));
			if (append)
				session.write(string + " -ec " + i + " -em " + remote.size());
			else
				session.write(string);
			i++;
		}
	}

	private void waitFor(String string, int i)
	{
		AtomicInteger ai = results.get(string);
		while (ai == null || ai.get() < i)
		{
			if (quit)
			{
				System.err.println("called exit2");
				try
				{
					finalize();
				}
				catch (Throwable e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				// System.exit(0);
				break;
			}
			Utils.sleep(1000);
			if (ai == null)
				ai = results.get(string);
		}
		results.clear();
	}

	@Override
	public void exceptionCaught(IoSession session, Throwable cause) throws Exception
	{
		System.err.println("Exepciton in " + session);
		cause.printStackTrace();
		// remove this node...
		if (remote.remove(session.getRemoteAddress()) != null)
			NODES--;
	}

	@Override
	public void sessionCreated(IoSession session) throws Exception
	{
		// timeout after 45min
		session.getConfig().setIdleTime(IdleStatus.BOTH_IDLE, 45 * 60);
		super.sessionCreated(session);
	}

	@Override
	public void sessionIdle(IoSession session, IdleStatus status) throws Exception
	{
		// remove this node...
		if (remote.remove(session.getRemoteAddress()) != null)
			NODES--;
	}

	@Override
	public void sessionClosed(IoSession session) throws Exception
	{
		// remove this node...
		if (remote.remove(session.getRemoteAddress()) != null)
			NODES--;
	}

	@Override
	public void messageReceived(IoSession session, Object message) throws Exception
	{
		if (quit)
			return;
		System.err.println("Remote node " + session.getRemoteAddress() + " got " + message);
		remote.put(session.getRemoteAddress(), session);
		String msg = (String) message;
		addResult(msg.trim());
		if (msg.trim().equals("ex"))
		{
			System.err.println("called exit");
			quit = true;
			for (IoSession sess : remote.values())
				sess.close(false);
			remote.clear();
			results.clear();
			// acceptor.unbind();
			System.err.println("called exit1");
		}
	}

	private synchronized void addResult(String msg)
	{
		AtomicInteger ai = results.get(msg);
		if (ai == null)
		{
			ai = new AtomicInteger(1);
			results.put(msg, ai);
		}
		else
			ai.incrementAndGet();
	}
}
