package ch.uzh.csg.filesharing;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.math.BigInteger;
import java.net.InetSocketAddress;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.atomic.AtomicInteger;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.PosixParser;
import org.apache.mina.core.future.ConnectFuture;
import org.apache.mina.core.service.IoConnector;
import org.apache.mina.core.service.IoHandlerAdapter;
import org.apache.mina.core.session.IoSession;
import org.apache.mina.filter.codec.ProtocolCodecFilter;
import org.apache.mina.filter.codec.textline.TextLineCodecFactory;
import org.apache.mina.transport.socket.nio.NioSocketConnector;

import ch.uzh.csg.filesharing.FileSharingNode.Strategy;
import ch.uzh.csg.filesharing.fileutils.FileUtils;
import ch.uzh.csg.filesharing.future.FutureFile;
import ch.uzh.csg.tomp2p.Configuration;
import ch.uzh.csg.tomp2p.Replication;
import ch.uzh.csg.tomp2p.Utils;
import ch.uzh.csg.tomp2p.common.BaseFutureAdapter;
import ch.uzh.csg.tomp2p.connection.BindInformation;
import ch.uzh.csg.tomp2p.connection.ConnectionHandler;
import ch.uzh.csg.tomp2p.connection.BindInformation.Protocol;
import ch.uzh.csg.tomp2p.routing.NodeAddress;

public class PSH2 extends IoHandlerAdapter
{
	final static private Options options = new Options();
	final private Options optionsCoordinator = new Options();
	static public FileSharingNode[] fileSharingNodes;
	final static private String DIRECTORY = System.getProperty("user.home") + File.separator
			+ ".FileSharing" + File.separator;
	static
	{
		options.addOption("c", true, "Coordinator");
		options.addOption("i", true, "Interface");
		options.addOption("p", true, "Port, 4001");
		options.addOption("s", true, "File");
		options.addOption("b", true, "Bootstrap");
		options.addOption("y", true, "Log File");
		FileUtils.turnSoftLinksOn();
	}

	public PSH2()
	{
		optionsCoordinator.addOption("scheme", true, "Incentive Scheme");
		optionsCoordinator.addOption("freeriders", true, "Freeriders in %");
		optionsCoordinator.addOption("limit", true, "Limit of trade");
		optionsCoordinator.addOption("credit", true, "Smallest credit to trade");
		optionsCoordinator.addOption("filemul", true, "Number of files, multiplicator of 29");
		optionsCoordinator.addOption("nodes", true, "Number of nodes");
		optionsCoordinator.addOption("machines", true, "Number of machines");
		optionsCoordinator.addOption("goodwill", true, "Goodwill in %");
		optionsCoordinator.addOption("ec", true, "Emanics count");
		optionsCoordinator.addOption("em", true, "Emanics total");
		optionsCoordinator.addOption("downloads", true, "Additional downoloads");
		optionsCoordinator.addOption("ttlPSH", true, "How old a value has to be to be traded");
	}

	private static int getInt(CommandLine cmd, String option, Integer defaul)
	{
		if (cmd.hasOption(option))
		{
			try
			{
				return Integer.parseInt(cmd.getOptionValue(option));
			}
			catch (NumberFormatException e)
			{
				throw new RuntimeException(cmd.getOptionValue(option)
						+ " is not an integer for option " + option);
			}
		}
		else if (defaul != null)
			return defaul;
		else
			throw new RuntimeException(option + " is mandatory");
	}

	private static String getString(CommandLine cmd, String option, String defaul)
	{
		if (cmd.hasOption(option))
			return cmd.getOptionValue(option);
		else if (defaul != null)
			return defaul;
		else
			throw new RuntimeException(option + " is mandatory");
	}

	private static double getDouble(CommandLine cmd, String option, Double defaul)
	{
		if (cmd.hasOption(option))
		{
			try
			{
				return Double.parseDouble(cmd.getOptionValue(option));
			}
			catch (NumberFormatException e)
			{
				throw new RuntimeException(cmd.getOptionValue(option)
						+ " is not a double for option " + option);
			}
		}
		else if (defaul != null)
			return defaul;
		else
			throw new RuntimeException(option + " is mandatory");
	}
	private static IoSession master;
	private static CommandLine cmd;

	public static void main(final String[] args) throws Exception
	{
		cmd = new PosixParser().parse(options, args);
		InetSocketAddress coordinator = ConversionUtils.createAddress(getString(cmd, "c", null));
		if (coordinator == null)
		{
			// we are local
			Coordinator.main(new String[] { "5000", "1" });
			coordinator = new InetSocketAddress("localhost", 5000);
		}
		IoConnector connector = new NioSocketConnector();
		connector.setHandler(new PSH2());
		connector.getFilterChain().addLast("codec",
				new ProtocolCodecFilter(new TextLineCodecFactory()));
		ConnectFuture cf = connector.connect(coordinator);
		System.err.println("Controller " + coordinator);
		cf.awaitUninterruptibly();
		master = cf.getSession();
		master.write("init");
	}
	ListFile lf;

	public void messageReceived(IoSession session, Object message) throws Exception
	{
		System.err.println("Coordinator: " + message);
		String msg = (String) message;
		if (msg.trim().equals("exit"))
		{
			ConnectionHandler.log("PSHx", "done!");
			FileSharingNode.dispose();
		}
		else if (msg.trim().equals("sim"))
		{
			simulate(lf);
			master.write("done sim");
		}
		else
		{
			if (fileSharingNodes != null)
			{
				for (int i = 0; i < fileSharingNodes.length; i++)
				{
					if (fileSharingNodes[i].isListening())
						fileSharingNodes[i].shutdown();
				}
			}
			Process p = Runtime.getRuntime().exec("rm -rf /home/uzh_fastss/.FileSharing");
			p.waitFor();
			p.destroy();
			p = Runtime.getRuntime().exec("rm -rf /tmp/fu");
			p.waitFor();
			p.destroy();
			String[] split = msg.split(" ");
			CommandLine cmdLine = new PosixParser().parse(optionsCoordinator, split);
			lf = bootstrap(cmdLine, cmd);
			master.write("done boot");
		}
	}

	private ListFile bootstrap(CommandLine cmdDynamic, CommandLine cmdStatic) throws Exception
	{
		int nodeNr = getInt(cmdDynamic, "nodes", 1);
		fileSharingNodes = new FileSharingNode[nodeNr];
		BindInformation bindInformation = new BindInformation();
		bindInformation.addListenProtocolHint(Protocol.IPv4);
		String iface = getString(cmdStatic, "i", "eth0");
		if (iface != null)
			bindInformation.addListenInterfaceHint(iface);
		int port = getInt(cmdStatic, "p", 4001);
		String file = getString(cmdStatic, "s", null);
		String strat = getString(cmdDynamic, "scheme", "PSH2");
		Strategy st;
		if (strat.toUpperCase().trim().equals("TFT"))
			st = Strategy.TFT;
		else if (strat.toUpperCase().trim().equals("PSHR"))
			st = Strategy.PSHR;
		else if (strat.toUpperCase().trim().equals("PSH2"))
			st = Strategy.PSH2;
		else if (strat.toUpperCase().trim().equals("NONE"))
			st = Strategy.NONE;
		else
			throw new RuntimeException("Unkown strategy " + strat);
		InetSocketAddress boot = ConversionUtils.createAddress(getString(cmdStatic, "b", null));
		String statFile = getString(cmdStatic, "y", "/tmp/stat.log");
		//
		double free = getDouble(cmdDynamic, "freeriders", 0d);
		int machines = getInt(cmdDynamic, "machines", 1);
		double goodwill = getDouble(cmdDynamic, "goodwill", 1d);
		int creditLimit = getInt(cmdDynamic, "limit", 4);
		int splitSize = getInt(cmdDynamic, "credit", 2);
		int ec = getInt(cmdDynamic, "ec", 0);
		int em = getInt(cmdDynamic, "em", 1);
		int mul = getInt(cmdDynamic, "filemul", 1);
		int downloads = getInt(cmdDynamic, "downloads", 1);
		int ttlPSH = getInt(cmdDynamic, "ttlPSH", 1);
		String s1 = "OPTIONS: Strategy:" + st + ", nodes:" + nodeNr + ", freeriders:" + free
				+ ", limit:" + creditLimit + ", credit:" + splitSize + " count:" + ec + ", total:"
				+ em + ", filemul:" + mul + ", dwn:" + downloads + ", machines: " + machines;
		System.err.println(s1);
		ConnectionHandler.log("PSH", s1);
		//
		ListFile lf = prepareFiles2(file, mul);
		lf.setNodeNr(nodeNr);
		lf.setDownloads(downloads);
		// boolean bootstrap = testPublish(bindInformation, boot.getAddress());
		prepareNodes(st, port, bindInformation, new File(statFile), boot, free, creditLimit,
				splitSize, lf.getFiles(), lf.getURNs(), goodwill, ttlPSH);
		initNodes(lf.getFiles(), lf.getURNs(), ec, em);
		for (int i = 0; i < fileSharingNodes.length; i++)
			fileSharingNodes[i].listen();
		Utils.sleep(5000);
		for (int i = 0; i < fileSharingNodes.length; i++)
		{
			System.err.println("bootstrap to1 " + boot);
			fileSharingNodes[i].bootstrap(boot).awaitUninterruptibly();
		}
		for (int i = 0; i < fileSharingNodes.length; i++)
		{
			Collection<NodeAddress> nas = fileSharingNodes[i].getRouting().getRandomNeighbors(1);
			if (nas.size() > 0)
				fileSharingNodes[i].bootstrap(nas.iterator().next()).awaitUninterruptibly();
		}
		// now we are bootstrapped
		for (int i = 0; i < fileSharingNodes.length; i++)
		{
			fileSharingNodes[i].afterBootstrap();
		}
		ConnectionHandler.log("PSH2", "bootstrap done");
		return lf;
	}

	private void simulate(ListFile lf) throws URISyntaxException, IOException
	{
		System.err.println("run simulation...");
		long start = System.currentTimeMillis();
		simulation(lf.getFiles(), lf.getAverage(), lf.getNodeNr(), lf.getDownloads());
		System.err.println(" Time: " + (System.currentTimeMillis() - start));
		System.err.println("shut down...");
		// FileSharingNode.dispose();
		System.err.println("done...");
	}

	private static ListFile prepareFiles2(String file, int mul) throws IOException,
			URISyntaxException
	{
		String line = null;
		BufferedReader in = new BufferedReader(new FileReader(file));
		List<String> fileList = new ArrayList<String>();
		List<URN> urns = new ArrayList<URN>();
		List<URN> popularity = new ArrayList<URN>();
		int i = 0;
		while ((line = in.readLine()) != null)
		{
			line = line.trim();
			String size = line.substring(0, line.indexOf("\t"));
			String nr = line.substring(line.indexOf("\t") + 1, line.lastIndexOf("\t"));
			String avg = line.substring(line.lastIndexOf("\t") + 1);
			double sz = Double.parseDouble(size);
			double num = Double.parseDouble(avg);
			for (int j = 0; j < mul; j++)
			{
				File f = FileUtils.generateRadomFile((int) sz, new File("/tmp/fu"), i, Utils
						.makeSHAHash("" + i));
				fileList.add(f.getName());
				URN urn = new URN(f.getName(), f.length(), Utils.makeSHAHash(f.getName()));
				urns.add(urn);
				for (int j1 = 0; j1 < num; j1++)
					popularity.add(urn);
				i++;
			}
			System.err.print(".");
		}
		System.err.println(".");
		System.err.println("created " + i + " files");
		in.close();
		return new ListFile(fileList, popularity, urns);
	}
	private static int ii = 0;

	private static void prepareNodes(Strategy st, int port, BindInformation bindInformation,
			File statFile, InetSocketAddress boot, double free, int creditLimit, int splitSize,
			List<String> files, List<URN> urns, double goodwill, int ttlPSH) throws IOException
	{
		if (statFile.exists())
		{
			FileUtils.copyFileHard(statFile,
					new File(statFile.getParent(), statFile.getName() + ii));
			ii++;
		}
		Utils.searchListenAddresses(bindInformation);
		int nrFreeRider = (int) (fileSharingNodes.length * free);
		for (int i = 0; i < fileSharingNodes.length; i++)
		{
			// BigInteger id = Utils.makeSHAHash("" + i + ""
			// + bindInformation.getListenAddresses().get(0));
			BigInteger id = Utils.createRandomNodeID();
			int pid = fileSharingNodes.length ^ ((int) (free * 10000)) ^ (st.ordinal() * 99)
					^ creditLimit;
			fileSharingNodes[i] = new FileSharingNode(pid, id, st, creditLimit, splitSize, files,
					urns, new File(DIRECTORY + "/" + id + "/published"), new File(DIRECTORY + "/"
							+ id + "/incoming"), ttlPSH * 1000, goodwill);
			if (i < nrFreeRider)
				fileSharingNodes[i].setFreeriding(true);
			fileSharingNodes[i].setListenDate(port, port, bindInformation, new Replication(false),
					statFile, boot);
		}
		Configuration.setConnectionTimeoutTCP(10000);
		Configuration.setConnectionTimeoutUDP(5000);
		System.err.println("outside " + bindInformation.getListenAddresses().get(0) + " boot: "
				+ boot.getAddress() + " eq:"
				+ boot.getAddress().equals(bindInformation.getListenAddresses().get(0)));
	}

	private static void initNodes(List<String> bag, List<URN> urns, int ec, int em)
			throws IOException, URISyntaxException
	{
		int fpp = bag.size() / em;
		int index = 0;
		for (int i = 0; i < fpp; i++)
		{
			index = (fpp * ec) + i;
			String f = bag.get(index);
			URN u = urns.get(index);
			fileSharingNodes[rnd.nextInt(fileSharingNodes.length)].addFile(new File("/tmp/fu", f),
					u);
			System.err.println("published " + u + " files.");
		}
		if (ec + 1 == em)
		{
			// last one
			for (int i = index + 1; i < bag.size(); i++)
			{
				String f = bag.get(i);
				URN u = urns.get(i);
				fileSharingNodes[rnd.nextInt(fileSharingNodes.length)].addFile(new File("/tmp/fu",
						f), u);
				System.err.println("published " + u + " files.");
			}
		}
		System.err.println("published " + bag.size() + " files.");
	}
	final private static Random rnd = new Random();

	private static Map<FileSharingNode, List<URN>> createDownload(List<URN> popularity,
			int downloads)
	{
		Map<FileSharingNode, List<URN>> dnMap = new HashMap<FileSharingNode, List<URN>>();
		for (int i = 0; i < fileSharingNodes.length; i++)
		{
			ArrayList<URN> al = new ArrayList<URN>();
			int index = rnd.nextInt(popularity.size());
			URN u = popularity.get(index);
			al.add(u);
			URN u2 = u;
			for (int x = 0; x < downloads; x++)
			{
				while (u2.equals(u))
				{
					index = rnd.nextInt(popularity.size());
					u = popularity.get(index);
				}
				al.add(u);
			}
			dnMap.put(fileSharingNodes[i], al);
		}
		return dnMap;
	}

	private static void simulation(final List<String> bag, List<URN> popularity, int nodes,
			int downloads) throws URISyntaxException, IOException
	{
		final AtomicInteger succ = new AtomicInteger(0);
		final AtomicInteger fail = new AtomicInteger(0);
		final AtomicInteger succFree = new AtomicInteger(0);
		final AtomicInteger failFree = new AtomicInteger(0);
		List<FutureFile> tmp = new ArrayList<FutureFile>();
		long tt = System.currentTimeMillis();
		Map<FileSharingNode, List<URN>> dnMap = createDownload(popularity, downloads);
		// List<FileSharingNode> notListenting = new
		// ArrayList<FileSharingNode>();
		// while (System.currentTimeMillis() - tt < 30 * 60 * 1000)
		{
			for (int i = 0; i < fileSharingNodes.length; i++)
			{
				fileSharingNodes[i].resetChunkCounter();
				System.err.println("download peer " + i);
				if (fileSharingNodes[i].isListening())
				{
					final List<URN> u1s = dnMap.get(fileSharingNodes[i]);
					int d = 0;
					for (final URN u1 : u1s)
					{
						ConnectionHandler.log("DWN", "" + d);
						d++;
						final int ix = i;
						if (!fileSharingNodes[i].isComplete(u1))
						{
							final FutureFile fileSharing = fileSharingNodes[ix].downloadURN(u1,
									30 * 60 * 1000);
							// fileSharing.awaitUninterruptibly();
							fileSharing.addListener(new BaseFutureAdapter<FutureFile>()
							{
								@Override
								public void operationComplete(FutureFile future)
								{
									System.err.println("download " + u1.getName() + ":"
											+ u1.getSize());
									count(fileSharing, fail, failFree, succ, succFree, bag,
											fileSharingNodes[ix].isFreerider());
								}
							});
							tmp.add(fileSharing);
						}
					}
				}
				/*
				 * if (i % 20 == 0 && i != 0) { for (FileSharingNode fsn :
				 * notListenting) fsn.listen(); notListenting.clear(); } for
				 * (int j = 0; j < nodes churn; j++) { int index = ((i + 1) (j +
				 * 1) 7777) % fileSharingNodes.length;
				 * fileSharingNodes[index].shutdown();
				 * notListenting.add(fileSharingNodes[index]); }
				 */
			}
			for (FutureFile fs : tmp)
			{
				fs.awaitUninterruptibly();
				if (fs.isFailed())
					System.err.println("Reason: " + fs.getFailedReason());
			}
			tmp.clear();
		}
		for (FutureFile fs : tmp)
		{
			fs.awaitUninterruptibly();
			if (fs.isFailed())
				System.err.println("Reason: " + fs.getFailedReason());
		}
		tmp.clear();
		//
		System.err.println("------------------------------");
		System.err.println("!!Success ratio: " + (succ.get() / ((double) fail.get() + succ.get()))
				+ "|time:" + (System.currentTimeMillis() - tt));
		System.err.println("!!Success ratio Free: "
				+ (succFree.get() / ((double) failFree.get() + succFree.get())) + "|time:"
				+ (System.currentTimeMillis() - tt));
		int counter = 0;
		for (int i = 0; i < fileSharingNodes.length; i++)
			counter += fileSharingNodes[i].countChunks();
		System.err.println("!!Chunks: " + counter);
		System.gc();
		System.err.print("\n");
	}

	private static void count(FutureFile future, AtomicInteger fail, AtomicInteger failFree,
			AtomicInteger succ, AtomicInteger succFree, List<String> bag, boolean free)
	{
		if (future.isCompleted())
		{
			if (future.isFailed())
			{
				System.err.print("-");
				if (free)
					failFree.incrementAndGet();
				else
					fail.incrementAndGet();
			}
			else
			{
				System.err.print("+");
				if (free)
					succFree.incrementAndGet();
				else
					succ.incrementAndGet();
			}
			System.err.println("success ratio: "
					+ (succ.get() / ((double) fail.get() + succ.get())));
		}
		else
		{
			System.err.println("WTF");
		}
	}
}

class ListFile
{
	final private List<String> files;
	final private List<URN> average;
	final private List<URN> urns;
	private int nodeNr;
	private int downloads;

	public ListFile(List<String> files, List<URN> average, List<URN> urns)
	{
		this.files = files;
		this.average = average;
		this.urns = urns;
	}

	public int getNodeNr()
	{
		return nodeNr;
	}

	public void setNodeNr(int nodeNr)
	{
		this.nodeNr = nodeNr;
	}

	public List<URN> getURNs()
	{
		return urns;
	}

	public List<String> getFiles()
	{
		return files;
	}

	public List<URN> getAverage()
	{
		return average;
	}

	public void setDownloads(int downloads)
	{
		this.downloads = downloads;
	}

	public int getDownloads()
	{
		return downloads;
	}
}