package ch.uzh.csg.filesharing.fastss;
import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;

import ch.uzh.csg.filesharing.ConversionUtils;
import ch.uzh.csg.filesharing.URN;

public class PreviewURN extends URN implements Comparable<PreviewURN>
{
	public static final long serialVersionUID = 4916209123296557348L;
	private String title;
	private String preview;
	private int hitCounter = 1;

	public PreviewURN(File file, String title, String preview) throws URISyntaxException
	{
		super(file);
		this.preview = preview;
		this.title = title;
	}

	public PreviewURN(URN urn, String title, String preview) throws URISyntaxException
	{
		super(urn.getName(), urn.getSize(), urn.getHash());
		this.preview = preview;
		this.title = title;
	}

	public PreviewURN(String urn) throws URISyntaxException
	{
		super(new URI(urn));
	}

	public String getPreview()
	{
		return preview;
	}

	public String getTitle()
	{
		return title;
	}

	public void incHit()
	{
		hitCounter++;
	}

	@Override
	protected void extensions(String nextToken)
	{
		if (nextToken.indexOf("p=") == 0)
		{
			byte[] me = ConversionUtils.decode(nextToken.substring(2).toCharArray());
			this.preview = new String(me);
		}
		else if (nextToken.indexOf("t=") == 0)
		{
			byte[] me = ConversionUtils.decode(nextToken.substring(2).toCharArray());
			this.title = new String(me);
		}
	}

	@Override
	public String toString()
	{
		StringBuilder sb = new StringBuilder(super.toString());
		sb.append(DELIM);
		sb.append("p=");
		byte[] data = preview.getBytes();
		sb.append(ConversionUtils.encode(data));
		sb.append(DELIM);
		sb.append("t=");
		data = title.getBytes();
		sb.append(ConversionUtils.encode(data));
		return sb.toString();
	}

	public String toShortString()
	{
		return super.toString();
	}

	public int compareTo(PreviewURN another)
	{
		if (equals(another))
			return 0;
		else
		{
			int diff = hitCounter - another.hitCounter;
			if (diff == 0)
				return getHash().compareTo(another.getHash());
			else
				return diff;
		}
	}
}
