package ch.uzh.csg.filesharing.fileutils;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.PrintWriter;
import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.List;

import ch.uzh.csg.filesharing.URN;
import ch.uzh.csg.tomp2p.Utils;

public class FileUtils
{
	private final static Object lock = new Object();
	private static Process p;
	private static PrintWriter pw;
	private static BufferedReader br;
	private static BufferedReader er;
	private static boolean softLink = false;
	private final static File TMP_DIR = new File("/tmp/fu");
	static
	{
		TMP_DIR.mkdirs();
	}

	public static void turnSoftLinksOn()
	{
		try
		{
			p = Runtime.getRuntime().exec("/bin/bash -i");
			pw = new PrintWriter(p.getOutputStream());
			br = new BufferedReader(new InputStreamReader(p.getInputStream()));
			er = new BufferedReader(new InputStreamReader(p.getErrorStream()));
			new Thread(new Runnable()
			{
				@Override
				public void run()
				{
					try
					{
						while (true)
						{
							char c = (char) br.read();
							// System.out.print(c);
						}
					}
					catch (IOException e)
					{
						e.printStackTrace();
					}
				}
			}).start();
			new Thread(new Runnable()
			{
				@Override
				public void run()
				{
					try
					{
						while (true)
						{
							char c = (char) er.read();
							// System.out.print(c);
						}
					}
					catch (IOException e)
					{
						e.printStackTrace();
					}
				}
			}).start();
			softLink = true;
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}

	public static File writeFile(String fileName, File localDirectory, byte[] download)
			throws IOException
	{
		if (!softLink)
			return writeFile(new File(localDirectory, fileName), download);
		else
			return writeFileSoft(fileName, localDirectory, download);
	}

	public static File writeFileSoft(String fileName, File localDirectory, byte[] download)
			throws IOException
	{
		synchronized (lock)
		{
			File linkName = new File(localDirectory, fileName);
			File targetName = new File(TMP_DIR, fileName);
			if (!targetName.exists())
				writeFile(targetName, download);
			createSoftLink(targetName, linkName);
			return linkName;
		}
	}

	public static File writeFile(File file, byte[] download) throws IOException
	{
		FileOutputStream fos = null;
		FileChannel fcout = null;
		try
		{
			fos = new FileOutputStream(file);
			fcout = fos.getChannel();
			for (int offset = 0; offset < download.length;)
			{
				int diff = download.length - offset;
				offset += fcout.write(ByteBuffer.wrap(download, offset, diff > 4096 ? 4096 : diff));
			}
		}
		finally
		{
			Utils.bestEffortclose(fcout, fos);
		}
		return file;
	}

	public static void copyFile(File in, File out) throws IOException
	{
		if (!softLink)
			copyFileHard(in, out);
		else
			copyFileSoft(in, out);
	}

	public static void copyFileHard(File in, File out) throws IOException
	{
		FileInputStream fis = null;
		FileOutputStream fos = null;
		FileChannel inChannel = null, outChannel = null;
		try
		{
			fis = new FileInputStream(in);
			inChannel = fis.getChannel();
			fos = new FileOutputStream(out);
			outChannel = fos.getChannel();
			inChannel.transferTo(0, inChannel.size(), outChannel);
		}
		finally
		{
			Utils.bestEffortclose(inChannel, fis, outChannel, fos);
		}
	}

	private static void copyFileSoft(File in, File out) throws IOException
	{
		createSoftLink(in, out);
	}

	public static File generateRadomFile(int size, File path, int prefix, BigInteger nodeID)
			throws IOException
	{
		path.mkdirs();
		File file = new File(path, "PSH_" + prefix + ".file");
		FileOutputStream fos = null;
		FileChannel fcout = null;
		byte[] me = new byte[4096];
		int rep = (me.length / 20);
		for (int i = 0; i < rep; i++)
			System.arraycopy(Utils.idToByteArray(nodeID), 0, me, i * 20, 20);
		try
		{
			fos = new FileOutputStream(file);
			fcout = fos.getChannel();
			for (int offset = 0; offset < size; offset += me.length)
			{
				ByteBuffer bb = ByteBuffer.wrap(me);
				if (offset + me.length > size)
					bb.limit(size - offset);
				fcout.write(bb);
			}
		}
		finally
		{
			Utils.bestEffortclose(fcout, fos);
		}
		return file;
	}

	public static void createSoftLink(File target, File link) throws IOException
	{
		if (!link.exists())
		{
			String cmd = createCommand(target, link);
			executeCommand(cmd);
		}
	}

	public static void createSoftLink(List<File> target, List<File> link) throws IOException
	{
		List<String> cmds = new ArrayList<String>();
		for (int i = 0; i < target.size() && i < link.size(); i++)
		{
			if (!link.get(i).exists())
				cmds.add(createCommand(target.get(i), link.get(i)));
		}
		executeCommand(cmds.toArray(new String[0]));
	}

	private static String createCommand(File target, File link)
	{
		StringBuilder sb = new StringBuilder("ln -s ");
		sb.append(target.toString());
		sb.append(" ");
		sb.append(link.toString());
		return sb.toString();
	}

	/*
	 * private synchronized static void executeCommand(String s) throws
	 * IOException { try { Process p = Runtime.getRuntime().exec(s);
	 * p.waitFor(); p.destroy(); } catch (InterruptedException e) {
	 * e.printStackTrace(); } }
	 */
	private static void executeCommand(String... cmds) throws IOException
	{
		if (cmds.length == 0)
			return;
		// final int MAX = 50;
		synchronized (pw)
		{
			// System.err.println("start...");
			for (int i = 0; i < cmds.length; i++)
			{
				// System.err.print(i + " ");
				pw.println(cmds[i]);
				pw.flush();
			}
			// System.err.println("stop...");
		}
	}

	@Override
	protected void finalize() throws Throwable
	{
		pw.println("exit");
		Utils.bestEffortclose(pw, br, er);
		p.waitFor();
		p.destroy();
		super.finalize();
	}

	public static boolean deleteDir(File dir)
	{
		if (dir.isDirectory())
		{
			String[] children = dir.list();
			for (int i = 0; i < children.length; i++)
			{
				boolean success = deleteDir(new File(dir, children[i]));
				if (!success)
				{
					return false;
				}
			}
		}
		// The directory is now empty so delete it
		return dir.delete();
	}

	public static List<URN> downloading(File[] metaFiles)
	{
		List<URN> urns = new ArrayList<URN>();
		FileInputStream in = null;
		ObjectInputStream s = null;
		for (File metaFile : metaFiles)
		{
			try
			{
				in = new FileInputStream(metaFile);
				s = new ObjectInputStream(in);
				URN urn = (URN) s.readObject();
				urns.add(urn);
			}
			catch (IOException e)
			{
			}
			catch (ClassNotFoundException e)
			{
			}
			finally
			{
				Utils.bestEffortclose(in, s);
			}
		}
		return urns;
	}
}
