package ch.uzh.csg.filesharing.fileutils;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;

import ch.uzh.csg.tomp2p.Utils;

public class MergerThread extends Observable implements Runnable
{
	// final private File splitFolder;
	final private List<File> splits;
	final private boolean deleteSegments;
	final private int noOfPieces;
	final private List<BigInteger> digestArray = new ArrayList<BigInteger>();
	final private File fileName;
	final private boolean digest;
	//
	private int progressCount = 0;
	private boolean finishedMerging = false;
	private Throwable error;

	public MergerThread(List<File> splits, File fileName, long fileLength, long splitSize,
			boolean deleteSegments, boolean digest)
	{
		this.splits = splits;
		this.fileName = fileName;
		this.deleteSegments = deleteSegments;
		this.noOfPieces = SplitterUtils.numPieces(fileLength, splitSize);
		this.digest = digest;
	}

	public int getNoOfPieces()
	{
		return noOfPieces;
	}

	public int getProgressCount()
	{
		return progressCount;
	}

	public boolean hasFinishedMerging()
	{
		return finishedMerging;
	}

	public boolean hasError()
	{
		return error != null;
	}

	public Throwable getError()
	{
		return error;
	}

	private void incrementProgressCount()
	{
		progressCount++;
		setChanged();
		notifyObservers();
	}

	public void merge()
	{
		SplitterUtils.getThreadPool().execute(this);
	}

	public void run()
	{
		try
		{
			mergeFile();
		}
		catch (IOException e)
		{
			System.err.println("Error while merging " + e);
			e.printStackTrace();
		}
	}

	public void mergeFile() throws IOException
	{
		File joinFile = fileName;
		FileInputStream fis = null;
		FileOutputStream fos = null;
		FileChannel fout = null;
		FileChannel fcin = null;
		List<File> deleteList = new ArrayList<File>(20);
		try
		{
			fos = new FileOutputStream(joinFile);
			fout = fos.getChannel();
			for (int index = 0; index < noOfPieces; index++)
			{
				File filepiece = splits.get(index);
				if (!(filepiece.exists()))
				{
					System.err.println("File " + filepiece + " not found");
					break;
				}
				fis = new FileInputStream(filepiece);
				fcin = fis.getChannel();
				fcin.transferTo(0, fcin.size(), fout);
				ByteBuffer bb = ByteBuffer.allocate((int) fcin.size());
				fcin.read(bb);
				if (digest)
				{
					bb.flip();
					digestArray.add(index, SplitterUtils.calculateDigest(bb));
				}
				fcin.close();
				fis.close();
				if (deleteSegments)
					deleteList.add(filepiece);
				incrementProgressCount();
			}
			if (deleteSegments)
				deleteSegments(deleteList);
		}
		finally
		{
			Utils.bestEffortclose(fout, fos, fcin, fis);
			finishedMerging = true;
			setChanged();
			notifyObservers();
		}
	}

	private void deleteSegments(List<File> deleteList)
	{
		for (File file : deleteList)
			file.delete();
	}
}
