package ch.uzh.csg.filesharing.fileutils;
import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.security.MessageDigest;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class SplitterUtils
{
	private final static MessageDigest md;
	private final static ExecutorService pool;
	static
	{
		try
		{
			md = MessageDigest.getInstance("SHA");
		}
		catch (Exception e)
		{
			e.printStackTrace();
			throw new RuntimeException("check your JVM for SHA");
		}
		pool = Executors.newFixedThreadPool(10);
	}

	public static BigInteger calculateDigest(ByteBuffer mbb)
	{
		byte[] dst = new byte[mbb.limit()];
		mbb.get(dst);
		md.update(dst);
		byte[] digest = md.digest();
		return new BigInteger(1, digest);
	}

	public static ExecutorService getThreadPool()
	{
		return pool;
	}

	public static int numPieces(long fileSize, long splitSize)
	{
		long noOfPieces = fileSize / splitSize;
		if ((noOfPieces * splitSize) != fileSize)
			noOfPieces++;
		if (noOfPieces != (int) noOfPieces)
			throw new RuntimeException("cannot create such a huge array");
		// System.err.println("pieces: " + noOfPieces + " for len:" + fileSize);
		return (int) noOfPieces;
	}
}
