package ch.uzh.csg.filesharing.future;
import ch.uzh.csg.filesharing.URN;
import ch.uzh.csg.tomp2p.common.BaseFutureImpl;
import ch.uzh.csg.tomp2p.message.Message;

public class FutureFileChunkImpl extends BaseFutureImpl implements FutureFileChunk
{
	final private URN urn;
	final private Progress progress;
	private Object hint;
	private int chunkID = -1;

	public FutureFileChunkImpl(URN urn, Progress progress)
	{
		synchronized (lock)
		{
			this.urn = urn;
			this.progress = progress;
		}
	}

	@Override
	public void chunkDownloaded(int chunkID)
	{
		synchronized (lock)
		{
			if (!setCompletedAndNotify())
				return;
			type = Message.TYPE_NOT_OK;
			this.chunkID = chunkID;
			progress.progressChanged();
		}
		notifyListerenrs();
	}

	@Override
	public URN getURN()
	{
		synchronized (lock)
		{
			return urn;
		}
	}

	@Override
	public void failed(String reason, Object hint)
	{
		synchronized (lock)
		{
			this.hint = hint;
			super.failed(Message.TYPE_NOT_OK, reason);
		}
	}

	@Override
	public void failed(Object hint)
	{
		failed("Unknown", hint);
	}

	@Override
	public Object getHint()
	{
		synchronized (lock)
		{
			return hint;
		}
	}

	@Override
	public int getChunkID()
	{
		synchronized (lock)
		{
			return chunkID;
		}
	}
}
