package ch.uzh.csg.filesharing.future;
import java.util.ArrayList;
import java.util.Collection;

import ch.uzh.csg.filesharing.FSProgressListener;
import ch.uzh.csg.filesharing.URN;
import ch.uzh.csg.filesharing.fileutils.FileChunkMapping;
import ch.uzh.csg.tomp2p.routing.NodeAddress;

public class FutureFileImpl extends FutureGenericFileImpl implements FutureFile
{
	private NodeAddress tracker;
	private double progress = 0d;
	private final Collection<FSProgressListener> listeners = new ArrayList<FSProgressListener>();
	private final URN urn;
	private final FileChunkMapping fileChunkMapping;

	public FutureFileImpl(URN urn, FileChunkMapping fileChunkMapping)
	{
		this.urn = urn;
		this.fileChunkMapping = fileChunkMapping;
	}

	@Override
	public void failed(NodeAddress tracker)
	{
		synchronized (lock)
		{
			if (!setCompletedAndNotify())
				return;
			this.tracker = tracker;
		}
		notifyListerenrs();
	}

	@Override
	public NodeAddress getTracker()
	{
		synchronized (lock)
		{
			return tracker;
		}
	}

	@Override
	public double getProgress()
	{
		synchronized (lock)
		{
			return progress;
		}
	}

	@Override
	public void progressChanged()
	{
		synchronized (lock)
		{
			double progress = fileChunkMapping.progress(urn);
			if (this.progress != progress)
			{
				this.progress = progress;
				for (FSProgressListener listener : listeners)
					listener.progress(urn, progress);
				resetTime();
			}
		}
	}

	@Override
	public void addProgressListener(FSProgressListener listener)
	{
		synchronized (lock)
		{
			listeners.add(listener);
		}
	}

	@Override
	public void removeProgressListener(FSProgressListener listener)
	{
		synchronized (lock)
		{
			listeners.remove(listener);
		}
	}

	@Override
	public URN getURN()
	{
		synchronized (lock)
		{
			return urn;
		}
	}
}
