package ch.uzh.csg.filesharing;
import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import java.net.URISyntaxException;
import java.nio.ByteBuffer;
import java.util.BitSet;

import org.junit.Assert;
import org.junit.Test;

import ch.uzh.csg.filesharing.fileutils.FileChunks;
import ch.uzh.csg.filesharing.fileutils.FileUtils;
import ch.uzh.csg.filesharing.fileutils.RandomBitSet;
import ch.uzh.csg.tomp2p.Utils;

public class TestChunks
{
	@Test
	public void testChunks() throws IOException, URISyntaxException
	{
		File f1 = FileUtils.generateRadomFile(20000, new File("/tmp"), 0, BigInteger.TEN);
		URN u = new URN(f1);
		FileChunks fc = new FileChunks(f1, u, 2000);
		//
		File incoming = new File("/tmp/i");
		incoming.mkdirs();
		FileChunks fc2 = new FileChunks(u, incoming, 2000);
		Assert.assertEquals(true, fc.isComplete());
		Assert.assertEquals(true, fc.isCompleting());
		//
		Assert.assertEquals(false, fc2.isComplete());
		Assert.assertEquals(false, fc2.isCompleting());
		//
		Assert.assertEquals(0, fc2.getStoredChunkSize());
		Assert.assertEquals(0, fc.getMissingChunkSize());
		//
		Assert.assertEquals(10, fc.getStoredChunkSize());
		Assert.assertEquals(10, fc2.getMissingChunkSize());
		// copy
		ByteBuffer bb = fc.getChunk(0);
		fc2.addChunk(0, bb);
		//
		Assert.assertEquals(1, fc2.getStoredChunkSize());
		Assert.assertEquals(9, fc2.getMissingChunkSize());
		//
		for (int i = 1; i < 10; i++)
		{
			bb = fc.getChunk(i);
			fc2.addChunk(i, bb);
		}
		Assert.assertEquals(10, fc2.getStoredChunkSize());
		Assert.assertEquals(0, fc2.getMissingChunkSize());
		Assert.assertEquals(true, fc2.isComplete());
		Assert.assertEquals(true, fc2.isCompleting());
		Assert.assertEquals(Utils.makeSHAHash(fc.getCompletFile()), Utils.makeSHAHash(fc2
				.getCompletFile()));
		fc.getCompletFile().delete();
		fc2.getCompletFile().delete();
	}

	@Test
	public void testChunks1() throws IOException, URISyntaxException
	{
		File f1 = FileUtils.generateRadomFile(20001, new File("/tmp"), 0, BigInteger.TEN);
		URN u = new URN(f1);
		FileChunks fc = new FileChunks(f1, u, 2000);
		//
		File incoming = new File("/tmp/i");
		incoming.mkdirs();
		FileChunks fc2 = new FileChunks(u, incoming, 2000);
		Assert.assertEquals(true, fc.isComplete());
		Assert.assertEquals(true, fc.isCompleting());
		//
		Assert.assertEquals(false, fc2.isComplete());
		Assert.assertEquals(false, fc2.isCompleting());
		//
		Assert.assertEquals(0, fc2.getStoredChunkSize());
		Assert.assertEquals(0, fc.getMissingChunkSize());
		//
		Assert.assertEquals(11, fc.getStoredChunkSize());
		Assert.assertEquals(11, fc2.getMissingChunkSize());
		// copy
		ByteBuffer bb = fc.getChunk(0);
		fc2.addChunk(0, bb);
		//
		Assert.assertEquals(1, fc2.getStoredChunkSize());
		Assert.assertEquals(10, fc2.getMissingChunkSize());
		//
		for (int i = 1; i < 11; i++)
		{
			bb = fc.getChunk(i);
			fc2.addChunk(i, bb);
		}
		Assert.assertEquals(11, fc2.getStoredChunkSize());
		Assert.assertEquals(0, fc2.getMissingChunkSize());
		Assert.assertEquals(true, fc2.isComplete());
		Assert.assertEquals(true, fc2.isCompleting());
		Assert.assertEquals(Utils.makeSHAHash(fc.getCompletFile()), Utils.makeSHAHash(fc2
				.getCompletFile()));
		fc.getCompletFile().delete();
		fc2.getCompletFile().delete();
	}

	@Test
	public void testChunks2() throws IOException, URISyntaxException
	{
		File f1 = FileUtils.generateRadomFile((20 * 1000 * 1000) - 2, new File("/tmp"), 0,
				BigInteger.TEN);
		URN u = new URN(f1);
		FileChunks fc = new FileChunks(f1, u, 2001);
		//
		File incoming = new File("/tmp/i");
		incoming.mkdirs();
		FileChunks fc2 = new FileChunks(u, incoming, 2001);
		Assert.assertEquals(true, fc.isComplete());
		Assert.assertEquals(true, fc.isCompleting());
		//
		Assert.assertEquals(false, fc2.isComplete());
		Assert.assertEquals(false, fc2.isCompleting());
		//
		Assert.assertEquals(0, fc2.getStoredChunkSize());
		Assert.assertEquals(0, fc.getMissingChunkSize());
		// copy
		ByteBuffer bb = fc.getChunk(0);
		fc2.addChunk(0, bb);
		//
		Assert.assertEquals(1, fc2.getStoredChunkSize());
		//
		for (int i = 1; !fc2.isCompleting(); i++)
		{
			bb = fc.getChunk(i);
			fc2.addChunk(i, bb);
		}
		Assert.assertEquals(0, fc2.getMissingChunkSize());
		Assert.assertEquals(true, fc2.isComplete());
		Assert.assertEquals(true, fc2.isCompleting());
		Assert.assertEquals(Utils.makeSHAHash(fc.getCompletFile()), Utils.makeSHAHash(fc2
				.getCompletFile()));
		fc.getCompletFile().delete();
		fc2.getCompletFile().delete();
	}

	@Test
	public void testRandomBitSet() throws IOException, URISyntaxException
	{
		BitSet bs = new BitSet(100);
		bs.set(50);
		RandomBitSet rbs = new RandomBitSet(bs);
		Assert.assertEquals(50, rbs.getRandomChunkNr());
		rbs.set(0);
		Assert.assertEquals(0, rbs.getRandomChunkNr());
		Assert.assertEquals(-1, rbs.getRandomChunkNr());
	}

	@Test
	public void testRandomBitSet1() throws IOException, URISyntaxException
	{
		BitSet bs = new BitSet(100);
		bs.set(0, 100);
		RandomBitSet rbs = new RandomBitSet(bs);
		for (int i = 0; i < 100; i++)
		{
			System.err.print(rbs.getRandomChunkNr() + ", ");
		}
		Assert.assertEquals(-1, rbs.getRandomChunkNr());
	}
}
