package ch.uzh.csg.filesharing;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Collection;
import java.util.Random;

import junit.framework.Assert;

import org.junit.Test;

import ch.uzh.csg.filesharing.FileSharingNode.Strategy;
import ch.uzh.csg.filesharing.future.FutureFile;
import ch.uzh.csg.filesharing.future.FutureFileImpl;
import ch.uzh.csg.filesharing.future.FuturePublish;
import ch.uzh.csg.filesharing.future.FutureURN;
import ch.uzh.csg.tomp2p.Utils;
import ch.uzh.csg.tomp2p.action.FutureDHT;
import ch.uzh.csg.tomp2p.action.FutureTracker;
import ch.uzh.csg.tomp2p.common.FutureResponse;
import ch.uzh.csg.tomp2p.message.Message;
import ch.uzh.csg.tomp2p.routing.NodeAddress;
import ch.uzh.csg.tomp2p.storage.ByteArrayObject;

public class TestFileSharing
{
	@Test
	public void testPublish() throws IOException, URISyntaxException
	{
		int nr = 10;
		FileSharingNode[] fsn = new FileSharingNode[nr];
		bootstrap(1, fsn);
		File f = createFile(20000);
		fsn[1].addFile(f);
		FuturePublish fp = fsn[1].addFileAndPublish(f);
		fp.awaitUninterruptibly();
		// Search filename
		FutureDHT ft = fsn[9].get(Utils.makeSHAHash(f.getName()), FileSharingNode.FILE_NAME);
		ft.awaitUninterruptibly();
		Assert.assertEquals(true, ft.isSuccess());
		URN urn = (URN) ft.getData().iterator().next();
		ft = fsn[9].get(urn.getHash(), FileSharingNode.TRACKERS);
		ft.awaitUninterruptibly();
		Assert.assertEquals(true, ft.isSuccess());
		NodeAddress na = (NodeAddress) ft.getData().iterator().next();
		Assert.assertEquals(fsn[1].getNodeAddress(), na);
		FutureTracker ft1 = fsn[9].getTracker().get(na, urn.getHash(), FileSharingNode.SEEDER);
		ft1.awaitUninterruptibly();
		Assert.assertEquals(fsn[1].getNodeAddress(), ft1.getRootTracker());
		for (int i = 0; i < 10; i++)
		{
			FutureResponse fr = fsn[9].getFileSharingRPC().get(ft1.getRootTracker(), urn.getHash(),
					i, null);
			fr.awaitUninterruptibly();
			Assert.assertEquals(true, fr.isSuccess());
			Message response = fr.getResponseMessage();
			Collection<ByteArrayObject> tmp = response.payloadAt(0);
			fsn[9].addChunk(urn, tmp, i);
		}
		Assert.assertEquals(true, fsn[9].isComplete(urn));
		System.err.println("comparing " + f + " and " + fsn[9].getCompleteFile(urn));
		Assert.assertEquals(Utils.makeSHAHash(f), Utils.makeSHAHash(fsn[9].getCompleteFile(urn)));
		close(fsn);
	}

	@Test
	public void testPublishAndDownload2() throws IOException, URISyntaxException
	{
		int nr = 10;
		FileSharingNode[] fsn = new FileSharingNode[nr];
		bootstrap(2, fsn);
		File f = createFile(20000);
		URN u = new URN(f);
		FuturePublish fp = fsn[1].addFileAndPublish(f);
		fp.awaitUninterruptibly();
		FutureFile ff = fsn[8].downloadURN(u);
		ff.awaitUninterruptibly();
		close(fsn);
	}

	@Test
	public void testPublishAndDownload3() throws IOException, URISyntaxException
	{
		int nr = 10;
		FileSharingNode[] fsn = new FileSharingNode[nr];
		bootstrap(3, fsn);
		File f = createFile(20000);
		URN u = new URN(f);
		FuturePublish fp = fsn[1].addFileAndPublish(f);
		fp.awaitUninterruptibly();
		FutureFile ff1 = fsn[8].downloadURN(u);
		FutureFile ff2 = fsn[2].downloadURN(u);
		FutureFile ff3 = fsn[5].downloadURN(u);
		ff1.awaitUninterruptibly();
		ff2.awaitUninterruptibly();
		ff3.awaitUninterruptibly();
		Assert.assertEquals(Utils.makeSHAHash(f), Utils.makeSHAHash(ff1.getFile()));
		Assert.assertEquals(Utils.makeSHAHash(f), Utils.makeSHAHash(ff2.getFile()));
		Assert.assertEquals(Utils.makeSHAHash(f), Utils.makeSHAHash(ff3.getFile()));
		close(fsn);
	}

	@Test
	public void testPublishAndDownload4() throws IOException, URISyntaxException
	{
		int nr = 20;
		FileSharingNode[] fsn = new FileSharingNode[nr];
		bootstrap(4, fsn);
		File f = createFile(20000);
		FuturePublish fp = fsn[1].addFileAndPublish(f);
		fp.awaitUninterruptibly();
		FutureURN fu1 = fsn[8].searchURN(f.getName());
		FutureURN fu2 = fsn[2].searchURN(f.getName());
		FutureURN fu3 = fsn[5].searchURN(f.getName());
		fu1.awaitUninterruptibly();
		fu2.awaitUninterruptibly();
		fu3.awaitUninterruptibly();
		FutureFile ff1 = fsn[8].downloadURN(fu1.getURNs().iterator().next());
		FutureFile ff2 = fsn[2].downloadURN(fu2.getURNs().iterator().next());
		FutureFile ff3 = fsn[5].downloadURN(fu3.getURNs().iterator().next());
		ff1.awaitUninterruptibly();
		ff2.awaitUninterruptibly();
		ff3.awaitUninterruptibly();
		Assert.assertEquals(Utils.makeSHAHash(f), Utils.makeSHAHash(ff1.getFile()));
		Assert.assertEquals(Utils.makeSHAHash(f), Utils.makeSHAHash(ff2.getFile()));
		Assert.assertEquals(Utils.makeSHAHash(f), Utils.makeSHAHash(ff3.getFile()));
		close(fsn);
	}

	@Test
	public void testPublishAndDownload5() throws IOException, URISyntaxException
	{
		int nr = 2;
		FileSharingNode[] fsn = new FileSharingNode[nr];
		File f1 = createFile(2 * 1000 * 1001);
		bootstrap(5, fsn);
		fsn[0].addFileAndPublish(f1).awaitUninterruptibly();
		URN u = new URN(f1);
		FutureFile ff = new FutureFileImpl(u, fsn[1].getFileChunkMapping());
		for (int i = 0; i < 11; i++)
			fsn[1].downloadChunk(u, i, fsn[0].getNodeAddress(), ff, false).awaitUninterruptibly();
	}

	private File createFile(int length) throws IOException
	{
		File f = File.createTempFile("testme", ".p2p");
		FileOutputStream fos = new FileOutputStream(f);
		Random rand = new Random();
		byte[] me = new byte[length];
		rand.nextBytes(me);
		fos.write(me);
		fos.close();
		return f;
	}

	private void bootstrap(int p2pId, FileSharingNode[] fsn) throws IOException
	{
		for (int i = 0; i < fsn.length; i++)
		{
			fsn[i] = new FileSharingNode(p2pId, Utils.makeSHAHash("" + (i + 1)), Strategy.NONE,
					2000 * 3, 2000, null, null, new File("/tmp/a"), new File("/tmp/b"), 0, 0);
			fsn[i].listen(2000, 2000);
			if (i != 0)
				fsn[i].bootstrap(fsn[0]);
		}
	}

	private void close(FileSharingNode[] fsn)
	{
		for (FileSharingNode fs : fsn)
			fs.shutdown();
	}
}
