package ch.uzh.csg.filesharing;
import java.math.BigInteger;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.util.Collection;

import junit.framework.Assert;

import org.junit.Test;

import ch.uzh.csg.tomp2p.Utils;
import ch.uzh.csg.tomp2p.routing.NodeAddress;
import ch.uzh.csg.tomp2p.storage.ByteArrayObject;

public class TestHistory
{
	private InetSocketAddress addr() throws UnknownHostException
	{
		InetAddress inetSend = InetAddress.getByName("127.0.0.1");
		return new InetSocketAddress(inetSend, 4000);
	}

	@Test
	public void testHistory() throws UnknownHostException
	{
		NodeAddress na1 = new NodeAddress(Utils.makeSHAHash("1"), addr(), addr());
		NodeAddress na2 = new NodeAddress(Utils.makeSHAHash("2"), addr(), addr());
		NodeAddress na3 = new NodeAddress(Utils.makeSHAHash("3"), addr(), addr());
		History h1 = new History(1, 1, 0);
		History h2 = new History(1, 1, 0);
		History h3 = new History(1, 1, 0);
		// na1 downloads from na2, 1
		if (h1.canDownloadFrom(na2, 1))
		{
			h1.increaseDownload(na2, 1);
			h2.increaseDownload(na1, -1);
		}
		else
			Assert.fail();
		// na2 downloads from na3, 1
		if (h2.canDownloadFrom(na3, 1))
		{
			h2.increaseDownload(na3, 1);
			h3.increaseDownload(na2, -1);
		}
		else
			Assert.fail();
		// na3 downloads from na1, 2, NP!
		if (h3.canDownloadFrom(na1, 2))
		{
			Assert.fail();
		}
		else
		{
			Collection<NodeAddress> c = h3.findPath(new byte[0], 1);
			Assert.assertEquals(0, c.size());
			byte[] bf = h1.bloomFilter();
			c = h3.findPath(bf, 1);
			System.err.println("BF: " + bf);
			Assert.assertEquals(1, c.size());
			// now ask this intermeditate peer
			NodeAddress inter = c.iterator().next();
			Assert.assertEquals(na2, inter);
			h2.account(na3, na1, 1);
			Check ch = new Check(na2, na1, 1);
			h1.applyCheckRecipient(na3, ch);
			h3.applyCheckSender(na1, ch);
			if (h3.canDownloadFrom(na1, 2))
			{
				h3.increaseDownload(na1, 2);
				h1.increaseDownload(na3, -2);
			}
			else
				Assert.fail();
		}
		if (h1.canDownloadFrom(na3, 2))
		{
			h1.increaseDownload(na3, 2);
			h3.increaseDownload(na1, -2);
		}
		else
			Assert.fail();
	}

	@Test
	public void testHistory2() throws UnknownHostException
	{
		History h1 = new History(1, 1, 0);
		for (int i = 0; i < 2000; i++)
		{
			NodeAddress na1 = new NodeAddress(Utils.makeSHAHash("" + i), addr(), addr());
			h1.increaseDownload(na1, 1);
		}
		byte[] me = h1.bloomFilter(1);
		Assert.assertEquals(1300, me.length);
	}

	@Test
	public void testHistory3() throws UnknownHostException
	{
		History h1 = new History(1, 1, 0);
		for (int i = 0; i < 2000; i++)
		{
			NodeAddress na1 = new NodeAddress(Utils.makeSHAHash("" + i), addr(), addr());
			h1.increaseDownload(na1, 1);
		}
		Collection<BigInteger> latestPeers = h1.getLatestPeers(1);
		Collection<ByteArrayObject> b = Utils.wrapCollection(latestPeers);
		System.err.println(latestPeers.size());
		// Assert.assertEquals(1298, Message.payloadLength(b));
	}
}
